/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Qt
#include <QtQml/QtQml>

// local
#include "application.h"
#include "application_manager.h"
#include "application_list_model.h"
#include "applicationscreenshotprovider.h"

// unity-mir
#include "logging.h"

static QObject* applicationManagerSingleton(QQmlEngine* engine, QJSEngine* scriptEngine) {
    Q_UNUSED(engine);
    Q_UNUSED(scriptEngine);
    DLOG("applicationManagerSingleton (engine=%p, scriptEngine=%p)", engine, scriptEngine);

    static ApplicationManager* instance = new ApplicationManager();
    return instance;
}

class UbuntuApplicationManagerPlugin : public QQmlExtensionPlugin {
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface/1.0")

    virtual void registerTypes(const char* uri)
    {
        DLOG("UbuntuApplicationManagerPlugin::registerTypes (this=%p, uri='%s')", this, uri);
        ASSERT(QLatin1String(uri) == QLatin1String("Unity.ApplicationManager"));

        qRegisterMetaType<ApplicationManager*>("ApplicationManager*"); //need for queueing signals

        qmlRegisterSingletonType<ApplicationManager>(
                    uri, 0, 1, "ApplicationManager", applicationManagerSingleton);
        qmlRegisterUncreatableType<Application>(
                    uri, 0, 1, "ApplicationInfo", "ApplicationInfo can't be instantiated");
        qmlRegisterUncreatableType<ApplicationListModel>(
                    uri, 0, 1, "ApplicationListModel", "ApplicationListModel can't be instantiated");
    }

    virtual void initializeEngine(QQmlEngine *engine, const char *uri)
    {
        QQmlExtensionPlugin::initializeEngine(engine, uri);

        ApplicationManager* appManager = static_cast<ApplicationManager*>(applicationManagerSingleton(engine, NULL));
        engine->addImageProvider(QLatin1String("screenshot"), new ApplicationScreenshotProvider(appManager));
    }
};

#include "plugin.moc"
